#include <windows.h>
#include <ddraw.h>
#include "ddutil.h"

#define  WindowTitle      "DirectDraw-Tutorial"
#define  WindowClassName  "DDWinClass"

LPDIRECTDRAW7			lpDirectDraw;	// DirectDraw-Objekt
HRESULT					ddrval;
LPDIRECTDRAWSURFACE7	lpddsPrimary;
LPDIRECTDRAWSURFACE7	lpddsBack;
LPDIRECTDRAWSURFACE7	lpddsBitmap;
DDSURFACEDESC2			ddsd;
DDSCAPS2				ddscaps; 
DDBLTFX					ddbltfx;
LPDIRECTDRAWCLIPPER		lpddClipper;
RECT					fill_area;
RECT					rectbitmap; 
bool					running;

void ddRelease ();

void AddColorKey (LPDIRECTDRAWSURFACE7 surface, DWORD Low, DWORD High)
{
	DDCOLORKEY key;
	ZeroMemory(&key, sizeof(key));
	key.dwColorSpaceLowValue = Low;
	key.dwColorSpaceHighValue = High;
	surface->SetColorKey (DDCKEY_SRCBLT, &key);
}

bool ddInit (HWND hwnd)
{
	ddrval = DirectDrawCreateEx(NULL, (VOID**)&lpDirectDraw, IID_IDirectDraw7, NULL); 
	if (ddrval != DD_OK)
	{
		return (0);
	}    

	ddrval = lpDirectDraw->SetCooperativeLevel (hwnd, DDSCL_EXCLUSIVE |	DDSCL_FULLSCREEN);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return (0);
	}

	ddrval = lpDirectDraw->SetDisplayMode (640, 480, 8, 0, 0);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return (0);
	}

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1; 
	ddrval = lpDirectDraw->CreateSurface (&ddsd, &lpddsPrimary, NULL);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return (0);
	}
	ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
	lpddsPrimary->GetAttachedSurface(&ddscaps, &lpddsBack); 

	ddrval = lpDirectDraw->CreateClipper(0, &lpddClipper, NULL);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return false;
	}

	struct 
	{
		RGNDATAHEADER rdh;
		RECT   rgndata[1];
	} cliplist;

	cliplist.rdh.dwSize   = sizeof (RGNDATAHEADER);
	cliplist.rdh.iType   = RDH_RECTANGLES;
	cliplist.rdh.nCount   = 1;
	cliplist.rdh.nRgnSize  = 0;
	cliplist.rdh.rcBound.left = 0;
	cliplist.rdh.rcBound.top = 0;
	cliplist.rdh.rcBound.right = 640;
	cliplist.rdh.rcBound.bottom = 480;
	cliplist.rgndata[0].left = 0;
	cliplist.rgndata[0].top  = 0;
	cliplist.rgndata[0].right = 640;
	cliplist.rgndata[0].bottom = 480;

	ddrval = lpddClipper->SetClipList ((LPRGNDATA) &cliplist, 0);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return false;
	}
	
	ddrval = lpddsBack->SetClipper(lpddClipper);
	if (ddrval != DD_OK)
	{
		ddRelease ();
		return false;
	}

	return true;
}

void ddRelease ()
{
	if (lpddClipper != NULL)
	{
		lpddClipper->Release();
		lpddClipper = NULL;
	} 

	if (lpddsBack != NULL)
	{
		lpddsBack->Release();
		lpddsBack = NULL;
	} 


	if (lpddsPrimary != NULL)
	{
		lpddsPrimary->Release();
		lpddsPrimary = NULL;
	}

	if (lpDirectDraw != NULL)
	{
		lpDirectDraw->Release();
		lpDirectDraw = NULL;
	}
}

void ddLastInits ()
{
	ddbltfx.dwSize = sizeof(DDBLTFX);
	ddbltfx.dwFillColor = RGB(0, 0, 0); 
	SetRect(&fill_area, 0, 0, 640, 480);

	lpddsBitmap = DDLoadBitmap(lpDirectDraw, "Test.bmp", 200, 150);
	AddColorKey(lpddsBitmap, 0, 0);
}

void ddDraw ()
{
	lpddsBack->Blt(&fill_area, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx); 
	SetRect(&rectbitmap, 220, 165, 420, 315); 
	lpddsBack->Blt(&rectbitmap, lpddsBitmap, NULL, DDBLT_WAIT, NULL);
	SetRect(&rectbitmap, 100, 100, 320, 250); 
	lpddsBack->Blt(&rectbitmap, lpddsBitmap, NULL, DDBLT_WAIT | DDBLT_KEYSRC, NULL);

	while(lpddsPrimary->Flip(NULL, DDFLIP_WAIT) != DD_OK); 
}


LONG FAR WINAPI WndProc (HWND , UINT , UINT , LONG) ;

int WINAPI WinMain (HINSTANCE hInstance , HINSTANCE hPrevInstance , 
					LPSTR lpCmdLine , int nCmdShow)
{
	MSG      msg      ;  
	HWND     hwnd     ;   
	WNDCLASS wndclass ; 
	if (!hPrevInstance)
    {
		wndclass.style         = CS_HREDRAW | CS_VREDRAW  ;
		wndclass.lpfnWndProc   = WndProc ;
		wndclass.cbClsExtra    = 0 ;
		wndclass.cbWndExtra    = 0 ;
		wndclass.hInstance     = hInstance   ;
		wndclass.hIcon         = LoadIcon (NULL , IDI_APPLICATION) ;
		wndclass.hCursor       = LoadCursor (NULL , IDC_ARROW)  ; 
		wndclass.hbrBackground = (HBRUSH)GetStockObject (LTGRAY_BRUSH)  ;
		wndclass.lpszMenuName  = NULL              ;
		wndclass.lpszClassName = WindowClassName ; 
		RegisterClass (&wndclass) ;    
	}
	hwnd = CreateWindow (WindowClassName , 
                         WindowTitle , 
                         WS_OVERLAPPEDWINDOW , 
			             CW_USEDEFAULT , 
						 CW_USEDEFAULT ,
                         CW_USEDEFAULT , 
                         CW_USEDEFAULT ,     
                         NULL , NULL , 
                         hInstance , NULL) ;     
	ShowWindow   (hwnd , nCmdShow) ;  
	UpdateWindow (hwnd) ;

	ddInit (hwnd);

	ddLastInits ();

	running=true;

	while(running)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			if (msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		ddDraw ();
	} 

	ddRelease ();

	return msg.wParam ;
}


LONG FAR WINAPI WndProc (HWND hwnd , UINT message , 
                         UINT wParam , LONG lParam) 
{ 
	switch (message)
	{
		case WM_KEYDOWN:
		{
			switch (wParam)
			{
			case VK_ESCAPE:
				running=false;
				return 0;
			} break;
		}
		case WM_DESTROY:
		{
			PostQuitMessage (0) ; 
			return 0 ;
		} break;
	} // switch (message)
	return DefWindowProc (hwnd , message , wParam , lParam) ;
} // WndProc 

